﻿(function() { "use strict"; // fake module


const current = Windows.UI.WebUI.WebUIBackgroundTaskInstance.current;
const localSettings = Windows.Storage.ApplicationData.current.localSettings.values;
const BackgroundDownloader = Windows.Networking.BackgroundTransfer.BackgroundDownloader;
const TileUpdateManager = Windows.UI.Notifications.TileUpdateManager;
const TileNotification = Windows.UI.Notifications.TileNotification;
const XmlDocument = Windows.Data.Xml.Dom.XmlDocument;
const assign = Object.assign;

const httpTimeout = 15 * 1000; // Wait this long for an HTTP response.
const maxLogQueue = 50; // Queue this many logEvent() calls; throw away oldest.

/**
 * Mimics a little bit of the class defined in 'errors/HttpError.js'.
 */
function HttpError(status, fields) {
    return assign(new Error(), fields, { name: `HttpError`, $code: `H${ status }` });
}

/**
 * Mimics a little bit of the class defined in 'errors/BackgroundError.js'.
 */
function BackgroundError(fields) {
    return assign(new Error(), fields, { name: `BackgroundError` });
}

function windowsErrorCode(number) {
    return `W${ (0x100000000 + number).toString(16).toUpperCase() }`;
}

function unknownTask(args) {
    const name = args.name;
    throw BackgroundError({ $code: `Q3`, message: `TASK_UNKNOWN`, debug: name });
}

function dispatchTask(args) {
    const name = args.name;
    args = args.args || {};
    return (knownTasks[name] || unknownTask)({ name, args });
}

function metaTask() {
    return dispatchTask(JSON.parse(current.triggerDetails.arguments.metaArgs));
}

function setResult(result) {
    localSettings.backgroundResult = JSON.stringify(result);
}

function getConfig() {
    return JSON.parse(localSettings.backgroundConfig || `{}`);
}

function setConfig(config) {
    localSettings.backgroundConfig = JSON.stringify(config);
}

function getLog() {
    return JSON.parse(localSettings.backgroundLog || `[]`);
}

function setLog(log) {
    localSettings.backgroundLog = JSON.stringify(log);
}

function logEvent(event) {
    try {
        const time = Date.now();
        const log = getLog().slice(1 - maxLogQueue);
        setLog(log.concat(assign({ time }, event)));
    }
    catch (ignore) {
        // Logging is best effort; ignore errors.
    }
}

function forwardLog() {
    const log = getLog();
    setLog([]);
    return log;
}

function resumeDownloads() {
    return (
        BackgroundDownloader
            .getCurrentDownloadsAsync()
            .then(downloads => {

                // Fire and forget each attach, since we're only doing this to "tickle"
                // the downloads, and don't actually want to wait for them to complete.

                for (let i = 0; i < downloads.length; ++i) {
                    downloads[i]
                        .attachAsync()
                        .catch(error => {
                            const number = error.number;
                            logEvent({
                                kind: `resumeDownloadsError`,
                                code: number < 0 ? windowsErrorCode(number) : number,
                                error: { stack: error.stack }
                            });
                        });
                }
            })
    );
}

function updateLiveTiles(args) {

    function windowsPleaseWork() {
        try {
            const tileUpdater = TileUpdateManager.createTileUpdaterForApplication();
            tileUpdater.enableNotificationQueue(true);
            return tileUpdater;
        }
        catch (ignore) {
            // Sometimes the API simply doesn't work, returning "The notification platform
            // is unavailable".  In this case, there is really nothing we can do.  We just
            // know this happens, and have decided to silently give up.
        }
    }

    function update(liveTiles) {
        tileUpdater.clear();
        for (let i = 0; i < liveTiles.length; ++i) {
            const xml = liveTiles[i].tile;
            const dom = new XmlDocument();
            dom.loadXml(xml);
            tileUpdater.update(new TileNotification(dom));
        }
    }

    const config = args.args;
    if (config) {
        setConfig(config);
    }

    const tileUpdater = windowsPleaseWork();
    if (!tileUpdater) {
        return;
    }

    return new Promise((resolve, reject) => {

        const falcorGet = assign(new XMLHttpRequest(), {
            onreadystatechange() {
                try {
                    if (falcorGet.readyState === XMLHttpRequest.DONE) {
                        const status = falcorGet.status;
                        switch (status) {
                            case 200 /* HTTP OK */: {
                                const falcorResponse = JSON.parse(falcorGet.response);
                                resolve(update(falcorResponse.jsonGraph.liveTiles.value));
                                break;
                            }
                            case 0 /* Probably just no network; silently ignore */ : {
                                resolve();
                                break;
                            }
                            default: {
                                const message = falcorGet.statusText;
                                const debug = config.appLanguages;
                                throw HttpError(status, { message, debug });
                            }
                        }
                    }
                }
                catch (error) {
                    reject(error);
                }
            },
            timeout: httpTimeout
        });

        const config = getConfig();

        const jsongUrl = (`
            https://${ config.edgeHost }/${ config.falcorRootPath }
                ? winx_lang=${ config.appLanguages }
                & falcor_server=0.1.0
                & method=get
                & path=["liveTiles"]
            `.replace(/\s+/g, ``)
        );

        falcorGet.open(`GET`, jsongUrl);
        falcorGet.send();
    });
}

const knownTasks = {
    metaTask,
    forwardLog,
    resumeDownloads,
    updateLiveTiles
};

function run(deferral) {
    Promise
        .resolve()
        .then(() =>
            dispatchTask({ name: current.task.name })
        )
        .then(value => {
            setResult({ value });
        })
        .catch(error => {
            const $code = error.$code;
            const stack = error.stack;
            const debug = error.debug;
            logEvent({
                kind: `backgroundTaskError`,
                code: $code,
                error: { stack, debug }
            });
            const name = error.name;
            const message = error.message;
            setResult({ error: { name, $code, message, stack, debug } });
        })
        .then(() => {
            deferral.complete();
            close();
        });
}

run(current.getDeferral());


})(); // fake module
